## Ejemplo con Pygame
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from math import cos, sin, pi, sqrt
from random import randint
from figura import Cilindro

def reshape((width, height)):
	if height == 0:
		height = 1
	glViewport(0, 0, width, height)
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	gluPerspective(45.0, float(width)/float(height), 0.1, 2000.0)
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()
	gluLookAt(  200.0, 200.0, 200.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0)

def init():
	glClearColor(0.0, 0.0, 0.0, 0.0)
	glShadeModel(GL_SMOOTH)
	glClearDepth(1.0)
	glEnable(GL_DEPTH_TEST)
	glDepthFunc(GL_LEQUAL)
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
	glEnable(GL_LIGHTING)
	glEnable(GL_LIGHT0)
	glLightiv(GL_LIGHT0, GL_POSITION, [ 0, 1000, 0, 1])

def display():
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT)
	glClearDepth(1.0)
	
	if wireframe:
		glPolygonMode(GL_FRONT, GL_LINE)
		glPolygonMode(GL_BACK, GL_LINE)
	else:
		glPolygonMode(GL_FRONT, GL_FILL)
		glPolygonMode(GL_BACK, GL_FILL)

	no_mat			= [ 0.0, 0.0, 0.0, 1.0 ]
	mat_ambient		= [ 0.7, 0.7, 0.7, 1.0 ]
	mat_ambient_color	= [ 0.8, 0.8, 0.2, 1.0 ]
	mat_diffuse		= [ 0.1, 0.5, 0.8, 1.0 ]
	mat_specular		= [ 1.0, 1.0, 1.0, 1.0 ]
	no_shininess		= [ 0 ]
	low_shininess		= [ 5 ]
	high_shininess		= [ 10 ]
	mat_emission		= [0.4, 0.2, 0.2, 0.0]
	
	glPushMatrix()
	glRotate( 45, 0.0, 1.0, 0.0)
	glRotate( 50, 1.0, 0.0, 0.0)
	
	glPushMatrix()
	glTranslate(-150.0, 0.0, 0.0)
	glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat)
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse)
	glMaterialfv(GL_FRONT, GL_SPECULAR, no_mat)
	glMaterialiv(GL_FRONT, GL_SHININESS, no_shininess)
	glMaterialfv(GL_FRONT, GL_EMISSION, no_mat)
	cilindro.drawOpenGL()
	glPopMatrix()
	
	glPushMatrix()
	glTranslate(-50.0, 0.0, 0.0)
	glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat)
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse)
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular)
	glMaterialiv(GL_FRONT, GL_SHININESS, low_shininess)
	glMaterialfv(GL_FRONT, GL_EMISSION, no_mat)
	cilindro.drawOpenGL()
	glPopMatrix()
	
	glPushMatrix()
	glTranslate(50.0, 0.0, 0.0)
	glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat)
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse)
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular)
	glMaterialiv(GL_FRONT, GL_SHININESS, high_shininess)
	glMaterialfv(GL_FRONT, GL_EMISSION, no_mat)
	cilindro.drawOpenGL()
	glPopMatrix()
	
	glPushMatrix()
	glTranslate(150.0, 0.0, 0.0)
	glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat)
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse)
	glMaterialfv(GL_FRONT, GL_SPECULAR, no_mat)
	glMaterialiv(GL_FRONT, GL_SHININESS, no_shininess)
	glMaterialfv(GL_FRONT, GL_EMISSION, mat_emission)
	cilindro.drawOpenGL()
	glPopMatrix()
	
	glPopMatrix()

def main():

	## estas tres llamadas son a pygame
	pygame.init()
	pygame.display.set_mode((600,400), HWSURFACE | OPENGL | DOUBLEBUF)
	pygame.display.set_caption("Clase Auxiliar 06")


	## estas dos llamadas son a funciones propias
	## configuran el ambiente y la escena
	reshape((600,400))
	init()
	# los objetos que dibujare
	global cilindro, tick, wireframe
	cilindro = Cilindro(40, 100, 180)

	wireframe = False
	continuar = True
	while continuar:
		tick = pygame.time.get_ticks()
		## Una manera de manejar eventos...
		for event in pygame.event.get():
			## para cerrar la ventana con la cruz del marco...
			if event.type == QUIT:
				continuar = False
			## para si se apreto algo del tecladoo
			if event.type == KEYDOWN:
				## para reconocer que tecla se apreto
				if (event.key == K_ESCAPE):
					continuar = False
				if (event.key == K_w):
					wireframe = not wireframe
		## esta es una llamada a una funcion propia
		display()
		## estas dos llamadas son a pygame
        	pygame.display.flip()

if __name__ == "__main__" :
	main ()
