from OpenGL.GL import *
from numpy import cross
from math import cos, sin, pi, sqrt

def calculaNormal(x0, y0, z0, x1, y1, z1, x2, y2, z2):
	Ux = x1-x0
	Uy = y1-y0
	Uz = z1-z0
	Vx = x2-x0
	Vy = y2-y0
	Vz = z2-z0

	Nx = Uy*Vz - Uz*Vy
	Ny = Uz*Vx - Ux*Vz
	Nz = Ux*Vy - Uy*Vx
 
	raiz = sqrt(Nx * Nx + Ny * Ny + Nz * Nz)
	return Nx/raiz, Ny/raiz, Nz/raiz

class Cilindro:
	def __init__(self, r, h, step=36):
		self.tapa1	= []
		self.tapa2	= []
		for i in range(step):
			self.tapa1.append( (r*cos(2*pi*i/step), r*sin(2*pi*i/step), h/2.0) )
			self.tapa2.append( (r*cos(2*pi*i/step), r*sin(2*pi*i/step), -h/2.0) )

	def drawOpenGL(self):
		n = len(self.tapa1)
		u =  self.tapa1
		d =	self.tapa2
		
		# el cuerpo
		for i in range(n):
			glBegin(GL_TRIANGLES)
			# la normal es importante!
			glNormal3fv( calculaNormal(u[i][0], u[i][1], u[i][2], d[i][0], d[i][1], d[i][2], d[(i+1)%n][0], d[(i+1)%n][1], d[(i+1)%n][2]) )
			glVertex3f( u[i][0], u[i][1], u[i][2])
			glVertex3f( d[i][0], d[i][1], d[i][2])
			glVertex3f( u[(i+1)%n][0], u[(i+1)%n][1], u[(i+1)%n][2])
			
			# la normal es importante!
			glNormal3fv( calculaNormal(d[i][0],d[i][1], d[i][2], d[(i+1)%n][0], d[(i+1)%n][1], d[(i+1)%n][2], u[(i+1)%n][0], u[(i+1)%n][1], u[(i+1)%n][2]) )
			glVertex3f( d[i][0], d[i][1], d[i][2])
			glVertex3f( d[(i+1)%n][0], d[(i+1)%n][1], d[(i+1)%n][2])
			glVertex3f( u[(i+1)%n][0], u[(i+1)%n][1], u[(i+1)%n][2])
			glEnd()
			
		# las tapas
		for i in range(n):
			glBegin(GL_TRIANGLES)
			# la normal es importante!
			glNormal3fv( calculaNormal(u[i][0],u[i][1], u[i][2], u[(i+1)%n][0], u[(i+1)%n][1], u[(i+1)%n][2], 0.0, 0.0, u[i][2]) )
			glVertex3f( u[i][0], u[i][1], u[i][2])
			glVertex3f( u[(i+1)%n][0], u[(i+1)%n][1], u[(i+1)%n][2])
			glVertex3f( 0.0, 0.0, u[i][2])
			
			# la normal es importante!
			glNormal3fv( calculaNormal( d[(i+1)%n][0], d[(i+1)%n][1], d[(i+1)%n][2],d[i][0],d[i][1], d[i][2], 0.0, 0.0, d[i][2] ) )
			glVertex3f( d[(i+1)%n][0], d[(i+1)%n][1], d[(i+1)%n][2])
			glVertex3f( d[i][0], d[i][1], d[i][2])
			glVertex3f( 0.0, 0.0, d[i][2])
			glEnd()
