#####################################################################
# CC3501-1
#####################################################################

import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from math import *
from Vector3D import *

#####################################################################

class Piramide:
    def __init__(self):
        self.lista = 0
        self.crear()
        
        self.dim = Vector(1.0,1.0,1.0) # dimensiones: ancho, alto y profundidad
        self.pos = Vector(0.0,0.0,0.0) # posicion de la piramide

    def escalar(self,dim):
        self.dim = dim

    def mover(self,d):
        self.pos = sumar(self.pos,d)

    def crear(self):
        p1=Vector(-0.5,-0.5,-0.5)
        p2=Vector( 0.5,-0.5,-0.5)
        p3=Vector( 0.5, 0.5,-0.5)
        p4=Vector(-0.5, 0.5,-0.5)
        p5=Vector( 0.0, 0.0, 0.5)
        
        self.lista = glGenLists(1)
        glNewList(self.lista,GL_COMPILE)

        glMaterial(GL_FRONT_AND_BACK, GL_SPECULAR, [0.01, 0.01, 0.01])
        glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE)
        glEnable(GL_COLOR_MATERIAL)
        
        glBegin(GL_TRIANGLES)
        
        glColor3f(1,1,0)
        dibujarTriangulo(p1,p2,p5)
        glColor3f(1,0,0)
        dibujarTriangulo(p2,p3,p5)
        glColor3f(0,1,0)
        dibujarTriangulo(p3,p4,p5)
        glColor3f(0,0,1)
        dibujarTriangulo(p4,p1,p5)
        glColor3f(1,1,1)
        dibujarCuadrilatero(p4,p3,p2,p1)

        glEnd()

        glEndList()

    def dibujar(self):
        glPushMatrix()
        
        glTranslatef(self.pos.x,self.pos.y,self.pos.z)
        glScalef    (self.dim.x,self.dim.y,self.dim.z)
        
        glCallList(self.lista)
        glPopMatrix()

    def actuar(self,dt):

        if control.up:
            self.mover(Vector(0,0,3*dt))
        if control.down:
            self.mover(Vector(0,0,-3*dt))

#####################################################################

def dibujarTriangulo(p1,p2,p3):
    glNormal3fv(normal(p1,p2,p3).cartesianas())
    glVertex3fv(p1.cartesianas())
    glVertex3fv(p2.cartesianas())
    glVertex3fv(p3.cartesianas())

def dibujarCuadrilatero(p1,p2,p3,p4):
    dibujarTriangulo(p1,p4,p3)
    dibujarTriangulo(p3,p2,p1)

#####################################################################

class Control:
    def __init__(self):
        self.up = False
        self.down = False

#####################################################################
# Funciones de graficos
#####################################################################

def reshape((width, height)):
    if height == 0:
        height = 1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(60, float(width)/float(height), 1, 2000.0)
    glMatrixMode(GL_MODELVIEW)

    # configuracion de camara
    eye=VectorEsfericas(1200.0,pi,2*pi/3)
    at=Vector(0.0, 0.0, 100.0)
    up=Vector(0.0, 0.0, 1.0)

    glLoadIdentity()
    gluLookAt(eye.x,eye.y,eye.z,
              at.x,at.y,at.z,
              up.x,up.y,up.z)

def init():
	glClearColor(0.0, 0.0, 0.0, 0.0)
	glShadeModel(GL_SMOOTH)
	glClearDepth(1.0)
	glEnable(GL_DEPTH_TEST)
	glDepthFunc(GL_LEQUAL)
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
	glEnable(GL_LIGHTING)
	
	glEnable(GL_LIGHT1)
	glLightfv(GL_LIGHT1, GL_POSITION, [ 100, 100.0, 300.0, 1.0])
	glLightfv(GL_LIGHT1, GL_AMBIENT,  [ 0.01, 0.01, 0.01])
	glLightfv(GL_LIGHT1, GL_SPECULAR, [ 10, 10, 10])
	glLightfv(GL_LIGHT1, GL_DIFFUSE,  [ 100, 100.0, 100.0, 1.0])

#####################################################################

def display(dt):
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    glClearDepth(1.0)

    for f in fs:
        f.actuar(dt)
        f.dibujar()
        
#####################################################################
# Programa principal
#####################################################################

def main(argv):

    global h,w,fs,control

    w=640
    h=480
    control=Control()

    pygame.init()
    pygame.display.set_mode((w,h), OPENGL|DOUBLEBUF)
    pygame.display.set_caption("Piramide")

    p=Piramide()
    p.escalar(Vector(300,300,300))

    # lista con todos los actores de la escena
    fs=[p]

    reshape((w,h))
    init()

    t0=pygame.time.get_ticks()

    run=True
    while run:
        t1=pygame.time.get_ticks()
        
        for event in pygame.event.get():
            if event.type == QUIT:
                run=False
            if event.type == KEYDOWN: #presionar tecla
                if event.key == K_ESCAPE:
                    run=False
                if event.key == K_UP:
                    control.up = True
                if event.key == K_DOWN:
                    control.down = True
            if event.type == KEYUP: #soltar tecla
                if event.key == K_UP:
                    control.up = False
                if event.key == K_DOWN:
                    control.down = False

        display((t1-t0)/10.0)
        t0=t1
        pygame.display.flip()
        pygame.time.wait(1000/30)

    pygame.quit()
    
if __name__ == "__main__":
    import sys
    main(sys.argv)

#####################################################################
