#####################################################################
# CC3501-1
#####################################################################

import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from math import *
from Vector3D import *

#####################################################################

class Piramide:
    def __init__(self):
        self.lista = 0
        self.crear()
        
        self.dim = Vector(1.0,1.0,1.0) # dimensiones: ancho, alto y profundidad
        self.pos = Vector(0.0,0.0,0.0) # posicion
        self.ang = Vector(0.0,0.0,0.0) # rotacion c/r a cada eje.

    def escalar(self,dim):
        self.dim = dim

    def mover(self,d):
        self.pos = sumar(self.pos,d)

    def rotar(self,a):
        self.ang = sumar(self.ang,a)

    def crear(self):
        p1=Vector(-0.5,-0.5,-0.5)
        p2=Vector( 0.5,-0.5,-0.5)
        p3=Vector( 0.5, 0.5,-0.5)
        p4=Vector(-0.5, 0.5,-0.5)
        p5=Vector( 0.0, 0.0, 0.5)
        
        self.lista = glGenLists(1)
        glNewList(self.lista,GL_COMPILE)

        glMaterial(GL_FRONT_AND_BACK, GL_SPECULAR, [0.01, 0.01, 0.01])
        glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE)
        glEnable(GL_COLOR_MATERIAL)
        
        glBegin(GL_TRIANGLES)
        
        glColor3f(1,1,0)
        dibujarTriangulo(p1,p2,p5)
        glColor3f(1,0,0)
        dibujarTriangulo(p2,p3,p5)
        glColor3f(0,1,0)
        dibujarTriangulo(p3,p4,p5)
        glColor3f(0,0,1)
        dibujarTriangulo(p4,p1,p5)
        glColor3f(1,1,1)
        dibujarCuadrilatero(p4,p3,p2,p1)

        glEnd()

        glEndList()

    def dibujar(self):
        glPushMatrix()
        
        glTranslatef(self.pos.x,self.pos.y,self.pos.z)
        glRotatef   (self.ang.x,1,0,0)
        glRotatef   (self.ang.y,0,1,0)
        glRotatef   (self.ang.z,0,0,1)
        glScalef    (self.dim.x,self.dim.y,self.dim.z)
        
        glCallList(self.lista)
        glPopMatrix()

    def actuar(self,dt):

        if control.up:
            self.mover(Vector(0,0,3*dt))
        if control.down:
            self.mover(Vector(0,0,-3*dt))

        #self.rotar(Vector(0,0,dt))

class Cilindro:
    def __init__(self):
        self.lista = 0
        self.crear()
        self.rgb = [1,1,1] # color inicial del cilindro
        
        self.dim = Vector(1.0,1.0,1.0) # dimensiones: ancho, alto y profundidad
        self.pos = Vector(0.0,0.0,0.0) # posicion 
        self.ang = Vector(0.0,0.0,0.0) # rotacion c/r a cada eje.

    def escalar(self,dim):
        self.dim = dim

    def mover(self,d):
        self.pos = sumar(self.pos,d)

    def rotar(self,a):
        self.ang = sumar(self.ang,a)

    def colorear(self,rgb):
        self.rgb = rgb

    def crear(self):
        a=Vector( 0.0, 0.0, 0.5)
        d=Vector( 0.0, 0.0,-0.5)
        
        self.lista = glGenLists(1)
        glNewList(self.lista,GL_COMPILE)

        glMaterial(GL_FRONT_AND_BACK, GL_SPECULAR, [0.01, 0.01, 0.01])
        glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE)
        glEnable(GL_COLOR_MATERIAL)
        
        glBegin(GL_TRIANGLES)
        
        ang=0.0
        dang=2*pi/50.0
        while ang<=2*pi:
            b=VectorCilindricas(0.5,ang, 0.5)
            c=VectorCilindricas(0.5,ang+dang,0.5)
            f=VectorCilindricas(0.5,ang,-0.5)
            e=VectorCilindricas(0.5,ang+dang,-0.5)
            dibujarTriangulo(a,b,c)
            dibujarTriangulo(d,e,f)
            dibujarCuadrilatero(c,b,f,e)
            ang=ang+dang            
            
        glEnd()

        glEndList()

    def dibujar(self):
        glPushMatrix()
        
        glTranslatef(self.pos.x,self.pos.y,self.pos.z)
        glRotatef   (self.ang.x,1,0,0)
        glRotatef   (self.ang.y,0,1,0)
        glRotatef   (self.ang.z,0,0,1)
        glScalef    (self.dim.x,self.dim.y,self.dim.z)
        
        glColor3f(self.rgb[0],self.rgb[1],self.rgb[2])
        glCallList(self.lista)
        glPopMatrix()

    def actuar(self,dt):

        if control.up:
            self.mover(Vector(0,0,-3*dt))
        if control.down:
            self.mover(Vector(0,0,3*dt))

        #self.rotar(Vector(0,dt,0))

#####################################################################

def dibujarTriangulo(p1,p2,p3):
    glNormal3fv(normal(p1,p2,p3).cartesianas())
    glVertex3fv(p1.cartesianas())
    glVertex3fv(p2.cartesianas())
    glVertex3fv(p3.cartesianas())

def dibujarCuadrilatero(p1,p2,p3,p4):
    dibujarTriangulo(p1,p4,p3)
    dibujarTriangulo(p3,p2,p1)

#####################################################################

class Control:
    def __init__(self):
        self.up = False
        self.down = False
        self.left = False
        self.right = False
        self.wireframe = False

class Camara:
    def __init__(self):
        # configuracion de camara, vista inicial
        self.eye = VectorEsfericas(1500.0,3*pi/4,pi/3)
        self.at = Vector(0.0, 0.0, 100.0)
        self.up = Vector(0.0, 0.0, 1.0)

    def actuar(self,dt):

        if control.left:
            self.at = sumar(self.eye,rotarFi(restar(self.at,self.eye),dt/200.0))

        if control.right:
            self.at = sumar(self.eye,rotarFi(restar(self.at,self.eye),-dt/200.0))
        
        glLoadIdentity()
        gluLookAt(self.eye.x,self.eye.y,self.eye.z,
              self.at.x,self.at.y,self.at.z,
              self.up.x,self.up.y,self.up.z)

#####################################################################
# Funciones de graficos
#####################################################################

def reshape((width, height)):
    if height == 0:
        height = 1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()

    gluPerspective(45, float(width)/float(height),1, 2000.0)
    #glOrtho(-w,w,-h,h,1,2000)
    
    glMatrixMode(GL_MODELVIEW)

def init():
	glClearColor(0.0, 0.0, 0.0, 0.0)
	glShadeModel(GL_SMOOTH)
	glClearDepth(1.0)
	glEnable(GL_DEPTH_TEST)
	glDepthFunc(GL_LEQUAL)
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
	glEnable(GL_LIGHTING)
	
	glEnable(GL_LIGHT1)
	glLightfv(GL_LIGHT1, GL_POSITION, [ 700, 700.0, 700.0, 1.0])
	glLightfv(GL_LIGHT1, GL_AMBIENT,  [ 0.2, 0.2, 0.2])
	glLightfv(GL_LIGHT1, GL_SPECULAR, [ 10, 10, 10])
	glLightfv(GL_LIGHT1, GL_DIFFUSE,  [ 200, 200.0, 200.0, 1.0])

	glEnable(GL_LIGHT2)
	glLightfv(GL_LIGHT2, GL_POSITION, [ -700, -700.0, 700.0, 1.0])
	glLightfv(GL_LIGHT2, GL_AMBIENT,  [ 0.2, 0.2, 0.2])
	glLightfv(GL_LIGHT2, GL_SPECULAR, [ 10, 10, 10])
	glLightfv(GL_LIGHT2, GL_DIFFUSE,  [ 200, 200.0, 200.0, 1.0])

#####################################################################

def display(dt):
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    glClearDepth(1.0)

    if control.wireframe:
        glPolygonMode(GL_FRONT, GL_LINE)
        glPolygonMode(GL_BACK, GL_LINE)
    else:
        glPolygonMode(GL_FRONT, GL_FILL)
        glPolygonMode(GL_BACK, GL_FILL)

    for f in fs:
        f.actuar(dt)
        f.dibujar()

    camara.actuar(dt)
        
#####################################################################
# Programa principal
#####################################################################

def main(argv):

    global h,w,fs,control,camara

    w=640
    h=480
    control=Control()
    camara=Camara()

    # genera ventana con pygame
    pygame.init()
    pygame.display.set_mode((w,h), OPENGL|DOUBLEBUF)
    pygame.display.set_caption("Piramide")

    p=Piramide()
    #p.mover(Vector(1,0,0))
    p.escalar(Vector(300,300,300))

    c=Cilindro()
    c.colorear([0,0,0.5])
    c.mover(Vector(100,0,0))
    c.escalar(Vector(300,300,300))

    c2=Cilindro()
    c2.colorear([0.5,0,0])
    c2.mover(Vector(-300,0,0))
    c2.escalar(Vector(300,300,30))

    # lista con todos los actores de la escena
    fs=[p,c,c2]

    # inicializa y configura OpenGL
    reshape((w,h))
    init()

    t0=pygame.time.get_ticks()

    run=True
    while run:
        t1=pygame.time.get_ticks()
        
        for event in pygame.event.get():
            if event.type == QUIT:
                run=False
            if event.type == KEYDOWN: #presionar tecla
                if event.key == K_ESCAPE:
                    run=False
                if event.key == K_w:
                    control.wireframe = not control.wireframe
                if event.key == K_UP:
                    control.up = True
                if event.key == K_DOWN:
                    control.down = True
                if event.key == K_LEFT:
                    control.left = True
                if event.key == K_RIGHT:
                    control.right = True
            if event.type == KEYUP: #soltar tecla
                if event.key == K_UP:
                    control.up = False
                if event.key == K_DOWN:
                    control.down = False
                if event.key == K_LEFT:
                    control.left = False
                if event.key == K_RIGHT:
                    control.right = False

        display((t1-t0)/10.0)
        t0=t1
        pygame.display.flip()
        pygame.time.wait(1000/30)

    # cierra la ventana
    pygame.quit()
    
if __name__ == "__main__":
    import sys
    main(sys.argv)

#####################################################################
