#####################################################################
# CC3501-1
#####################################################################

import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from math import *
from Vector3D import *

#####################################################################

class Cilindro:
    def __init__(self):
        self.lista = 0
        self.crear()
        self.rgb = [1,1,1] # color inicial del cilindro
        
        self.dim = Vector(1.0,1.0,1.0) # dimensiones: ancho, alto y profundidad
        self.pos = Vector(0.0,0.0,0.0) # posicion 
        self.ang = Vector(0.0,0.0,0.0) # rotacion c/r a cada eje.

    def escalar(self,dim):
        self.dim = dim

    def mover(self,d):
        self.pos = sumar(self.pos,d)

    def rotar(self,a):
        self.ang = sumar(self.ang,a)

    def colorear(self,rgb):
        self.rgb = rgb

    def crear(self):
        a=Vector( 0.0, 0.0, 0.5)
        d=Vector( 0.0, 0.0,-0.5)
        
        self.lista = glGenLists(1)
        glNewList(self.lista,GL_COMPILE)

        glMaterial(GL_FRONT_AND_BACK, GL_SPECULAR, [0.01, 0.01, 0.01])
        glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE)
        glEnable(GL_COLOR_MATERIAL)
        
        glBegin(GL_TRIANGLES)
        
        ang=0.0
        dang=2*pi/20.0
        while ang<=2*pi:
            b=VectorCilindricas(0.5,ang, 0.5)
            c=VectorCilindricas(0.5,ang+dang,0.5)
            f=VectorCilindricas(0.5,ang,-0.5)
            e=VectorCilindricas(0.5,ang+dang,-0.5)
            dibujarTriangulo(a,b,c)
            dibujarTriangulo(d,e,f)
            dibujarCuadrilatero(c,b,f,e)
            ang=ang+dang            
            
        glEnd()

        glEndList()

    def dibujar(self):
        glPushMatrix()
        
        glTranslatef(self.pos.x,self.pos.y,self.pos.z)
        glRotatef   (self.ang.x,1,0,0)
        glRotatef   (self.ang.y,0,1,0)
        glRotatef   (self.ang.z,0,0,1)
        glScalef    (self.dim.x,self.dim.y,self.dim.z)
        
        glColor3f(self.rgb[0],self.rgb[1],self.rgb[2])
        glCallList(self.lista)
        glPopMatrix()

    def actuar(self,dt):

        if control.up:
            self.mover(Vector(0,0,-3*dt))
        if control.down:
            self.mover(Vector(0,0,3*dt))

        self.rotar(Vector(0,dt,0))

class Mesa:
    def __init__(self):
        self.dim = Vector(1.0,1.0,1.0) # dimensiones: ancho, alto y profundidad
        self.pos = Vector(0.0,0.0,0.0) # posicion 
        self.ang = Vector(0.0,0.0,0.0) # rotacion c/r a cada eje.

        self.cubierta=Cilindro()
        self.pata1=Cilindro()
        self.pata2=Cilindro()
        self.pata3=Cilindro()
        self.pata4=Cilindro()

        self.cubierta.escalar(Vector(1,1,0.1))
        self.pata1.escalar(Vector(0.1,0.1,0.9))
        self.pata2.escalar(Vector(0.1,0.1,0.9))
        self.pata3.escalar(Vector(0.1,0.1,0.9))
        self.pata4.escalar(Vector(0.1,0.1,0.9))

        self.cubierta.pos=Vector(0,0,0.45)
        self.pata1.pos=Vector(0.3,0.3,-0.05)
        self.pata2.pos=Vector(0.3,-0.3,-0.05)
        self.pata3.pos=Vector(-0.3,-0.3,-0.05)
        self.pata4.pos=Vector(-0.3,0.3,-0.05)
        
    def escalar(self,dim):
        self.dim = dim

    def mover(self,d):
        self.pos = sumar(self.pos,d)

    def rotar(self,a):
        self.ang = sumar(self.ang,a)

    def colorear(self,rgb):
        self.cubierta.rgb = rgb
        self.pata1.rgb = rgb
        self.pata2.rgb = rgb
        self.pata3.rgb = rgb
        self.pata4.rgb = rgb

    def dibujar(self):
        glPushMatrix()
        
        glTranslatef(self.pos.x,self.pos.y,self.pos.z)
        glRotatef   (self.ang.x,1,0,0)
        glRotatef   (self.ang.y,0,1,0)
        glRotatef   (self.ang.z,0,0,1)
        glScalef    (self.dim.x,self.dim.y,self.dim.z)
                
        self.cubierta.dibujar()
        self.pata1.dibujar()
        self.pata2.dibujar()
        self.pata3.dibujar()
        self.pata4.dibujar()
        
        glPopMatrix()

    def actuar(self,dt):

        if control.up:
            self.rotar(Vector(0,dt,0))
        if control.down:
            self.rotar(Vector(0,-dt,0))

#####################################################################

def dibujarTriangulo(p1,p2,p3):
    glNormal3fv(normal(p1,p2,p3).cartesianas())
    glVertex3fv(p1.cartesianas())
    glVertex3fv(p2.cartesianas())
    glVertex3fv(p3.cartesianas())

def dibujarCuadrilatero(p1,p2,p3,p4):
    dibujarTriangulo(p1,p4,p3)
    dibujarTriangulo(p3,p2,p1)

#####################################################################

class Control:
    def __init__(self):
        self.up = False
        self.down = False
        self.left = False
        self.right = False
        self.wireframe = False
        self.verSR = False

class Camara:
    def __init__(self):
        # configuracion de camara, vista inicial
        self.eye = VectorEsfericas(1500.0,pi/4,pi/3)
        self.at = Vector(0.0, 0.0, 100.0)
        self.up = Vector(0.0, 0.0, 1.0)

    def actuar(self,dt):

        if control.left:
            self.at = sumar(self.eye,rotarFi(restar(self.at,self.eye),dt/200.0))

        if control.right:
            self.at = sumar(self.eye,rotarFi(restar(self.at,self.eye),-dt/200.0))
        
        glLoadIdentity()
        gluLookAt(self.eye.x,self.eye.y,self.eye.z,
              self.at.x,self.at.y,self.at.z,
              self.up.x,self.up.y,self.up.z)

    def dibujar(self):
        a=0

class Eje:
    def __init__(self, largo):
        self.largo =  largo

    def actuar(self,dt):
        a=0
        
    def dibujar(self):
        if control.verSR:
            # almaceno la matriz, para aplicar los cambios solo sobre el Eje
            glPushMatrix()
            glBegin(GL_LINES)
            # eje Y verde
            glColor4f( 0.0,   1.0, 0.0, 0.0)
            glVertex4f(0.0, self.largo, 0.0, 1.0)
            glVertex4f(0.0,   0.0, 0.0, 1.0)
            # eje x rojo
            glColor4f( 1.0  , 0.0, 0.0, 0.0)
            glVertex4f(0.0  , 0.0, 0.0, 1.0)
            glVertex4f(self.largo, 0.0, 0.0, 1.0)
            #eje z azul
            glColor4f( 0.0, 0.0, 1.0, 0.0)
            glVertex4f(0.0, 0.0,   0.0, 1.0)
            glVertex4f(0.0, 0.0, self.largo, 1.0)
            glEnd()
            #importante, recupero la matriz!
            glPopMatrix()

#####################################################################
# Funciones de graficos
#####################################################################

def reshape((width, height)):
    if height == 0:
        height = 1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()

    gluPerspective(45, float(width)/float(height),1, 2000.0)
    #glOrtho(-w,w,-h,h,1,2000)
    
    glMatrixMode(GL_MODELVIEW)

def init():
	glClearColor(0.0, 0.0, 0.0, 0.0)
	glShadeModel(GL_SMOOTH)
	glClearDepth(1.0)
	glEnable(GL_DEPTH_TEST)
	glDepthFunc(GL_LEQUAL)
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
	glEnable(GL_LIGHTING)
	
	glEnable(GL_LIGHT1)
	glLightfv(GL_LIGHT1, GL_POSITION, [ 700, 700.0, 700.0, 1.0])
	glLightfv(GL_LIGHT1, GL_AMBIENT,  [ 0.2, 0.2, 0.2])
	glLightfv(GL_LIGHT1, GL_SPECULAR, [ 10, 10, 10])
	glLightfv(GL_LIGHT1, GL_DIFFUSE,  [ 10, 10.0, 10.0, 1.0])

#####################################################################

def display(dt):
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    glClearDepth(1.0)

    if control.wireframe:
        glPolygonMode(GL_FRONT, GL_LINE)
        glPolygonMode(GL_BACK, GL_LINE)
    else:
        glPolygonMode(GL_FRONT, GL_FILL)
        glPolygonMode(GL_BACK, GL_FILL)

    for f in fs:
        f.actuar(dt)
        f.dibujar()
        
#####################################################################
# Programa principal
#####################################################################

def main(argv):

    global h,w,fs,control

    w=640
    h=480
    control=Control()
    camara=Camara()
    eje=Eje(1000)

    # genera ventana con pygame
    pygame.init()
    pygame.display.set_mode((w,h), OPENGL|DOUBLEBUF)
    pygame.display.set_caption("Piramide")

    m=Mesa()
    #m.colorear([1,0,0])
    m.escalar(Vector(500,500,700))

    # lista con todos los actores de la escena
    fs=[m,camara,eje]

    # inicializa y configura OpenGL
    reshape((w,h))
    init()

    t0=pygame.time.get_ticks()

    run=True
    while run:
        t1=pygame.time.get_ticks()
        
        for event in pygame.event.get():
            if event.type == QUIT:
                run=False
            if event.type == KEYDOWN: #presionar tecla
                if event.key == K_ESCAPE:
                    run=False
                if event.key == K_w:
                    control.wireframe = not control.wireframe
                if event.key == K_s:
                    control.verSR = not control.verSR
                if event.key == K_UP:
                    control.up = True
                if event.key == K_DOWN:
                    control.down = True
                if event.key == K_LEFT:
                    control.left = True
                if event.key == K_RIGHT:
                    control.right = True
            if event.type == KEYUP: #soltar tecla
                if event.key == K_UP:
                    control.up = False
                if event.key == K_DOWN:
                    control.down = False
                if event.key == K_LEFT:
                    control.left = False
                if event.key == K_RIGHT:
                    control.right = False

        display((t1-t0)/10.0)
        t0=t1
        pygame.display.flip()
        pygame.time.wait(1000/30)

    # cierra la ventana
    pygame.quit()
    
if __name__ == "__main__":
    import sys
    main(sys.argv)

#####################################################################
